import sys
import re
from pprint import pprint
import chardet

from tklib.tkfile import tkFile
import tklib.tkre as tkre


infile  = 'test.ini'
outfile = 'test-out.ini'

class tkIniFile(tkFile):
    """ 
    tklib Inifile class
    """

    var = {}
    # args: ProgramPath, UseAsIniFile, CreateIniFile
    def __init__(self, path = None, mode = 'r', OpenFile = 1, IsPrint = True, **args):
        super().__init__(path, mode, OpenFile, IsPrint = IsPrint, **args)
#        self.fp   = None
#        self.path = None
#        self.mode = 'r'
#        self.path = self.IfYes(path is not None, path, self.path)
#        self.mode = self.IfYes(mode is not None, mode, self.mode)

        self.KeyHead       = "System/"
        self.SystemKeyHead = self.KeyHead
        self.ProgramPath   = ''
        self.CreateIniFile = 0
        self.UseAsIniFile  = 0
        self.update(**args)

        self.var[self.KeyHead + "UseAsIniFile"]     = self.UseAsIniFile;
        self.var[self.KeyHead + "CreateIniFile"]    = self.CreateIniFile;
        self.var[self.SystemKeyHead +"ProgramFile"] = self.ProgramPath;

# this called from super().__init__
#        if self.path:
#            self.SetPath(self.path, self.mode)
#            self.Open(self.path, self.mode)

    def __del__(self):
        self.close()
        super(tkIniFile, self).__del__()

    def __str__(self):
        return self.ClassPath()


    def ReadAll(self, path = None, AddSection = 0, IsPrint = False, encoding = None):
        if not self.fp or path is not None:
            self.close()

            self.open(path = path, mode = 'r', IsPrint = IsPrint, encoding = encoding)

        if not self.fp:
            return None

        inf = {}
        secname = ''
        while 1:
            line = self.ReadLine()
            if not line:
                return inf
            if line == '':
                continue
            if tkre.Match('#', line):
                continue

            match = re.match(r'\s*\[(.*?)]', line)
            if match:
                secname = match.group(1)
                continue

            list = tkre.Match(r'\s*(\S.*?)\s*=\s*(.*)\s*$', line)
            if list is not None and len(list) >= 2:
                if AddSection:
                    key = secname + '/' + list[1]
                else:
                    key = list[1]
                val = list[2]
                inf[key] = val
#                print("{}: {}".format(key, val))

        self.close()

        return inf

    def GetNextSection(self):
        lines = []
        line = self.ReadLine()
        if line == '':
            return '', lines
#        print("l1: ", line, end='')
        match = re.match(r'\s*\[(.*?)]', line)
        if match:
            secname = match.group(1)
        else:
            secname = ''
        lines.append(line)
#        print("sec [{}]".format(secname))
        while 1:
            pos = self.Tell()
            line = self.ReadLine()
#            print("l2: ", line, end='')
            if line == '':
                break
            if re.match(r'\s*\[', line):
                self.Seek(pos)
                break
            lines.append(line)
        return secname, lines
        
    def FindSectionFromStr(self, section, lines = None):
        if lines is None:
            lines = self.lines

        i0   = None
# insert point: the last non-blank line in the section
        iins = None
        i1   = None
        for i in range(len(lines)):
            regsection = re.sub(r'([\[\]\(\)])', r'\\\1', section)
            if tkre.Match(r'\s*\[{}\]'.format(regsection), lines[i]):
                i0 = i
                break
        if i0 is not None:
            i1 = len(lines)
            for i in range(i0+1, len(lines)):
                if tkre.Match(r'\s*\[(.*?)]', lines[i]):
                    i1 = i - 1
                    break
# the insertion point is the last non-blank line in the section
                if tkre.Search(r'\S', lines[i]):
                    iins = i
#if the sction has no key, iins will be the next line to the section title
            if iins is None:
                iins = i1
            return i0, iins, i1 - 1
        return None, None, None

    def FindLastNonBlankLine(self, lines = None):
        if lines is None:
            lines = self.lines

        n = len(lines)
        il = 0
        for i in range(0, n):
            if tkre.Search(r'\S', lines[i]):
                il = i
        return il

    def FindKeyFromSectionRange(self, i0, i1, key, lines = None):
        if lines is None:
            lines = self.lines

        n = len(lines)
        if i1+1 > n:
            i1 = n - 1
#        print("range: ", i0, i1)
        for i in range(i0, i1+1):
#            print("i=", i, "  key=", key)
#            print(lines[i])
            regkey = re.sub(r'([\[\]\(\)])', r'\\\1', key)
#            print("s, reg=", key, regkey)
#            exit()
            if tkre.Match(r'\s*{}\s*='.format(regkey), lines[i]):
#                print("hit")
                return i
        return None

    def WriteString(self, section, key, value, outfile = None, IsPrint = False):
        return self.write_string(section, key, value, outfile = outfile, is_print = IsPrint)

    def write_string(self, section, key, value, outfile = None, is_print = False):
        debug = 0

        lines = []
        if not self.open(path = self.path, IsPrint = is_print):
            if is_print:
                print("Warning in tkinifile.WriteString: Can not read [{}]".format(self.path))
        else:
            lines = self.ReadLineList()
            self.close()

        i0, inotblank, i1 = self.FindSectionFromStr(section, lines)
        if debug:
            print(f"190 section={section}: key={key}: i0={i0} - inotblank={inotblank} - i1={i1}")

# Section is not found
        if i0 is None:
            il = self.FindLastNonBlankLine(lines)
            if debug:
#                print(f"{lines=}")
                print(f"197 i0 is None: il={il}", il)
# if file includes some data, add the new section with a blank line
            if il is not None and il > 0:
                if debug:
                    print(f"201 i0 is None: il={il}")
                lines.insert(il+1, "\n")
                il += 1
# add the new section
            lines.insert(il+1, "[{}]\n".format(section))
            lines.insert(il+2, "{}={}\n".format(key, value))
# section is found
        else:
            ik = self.FindKeyFromSectionRange(i0, inotblank, key, lines)
            if debug:
                print(f"211  key={key} ik={ik}")
# key is not found, will be added to the next line
            if ik is None:
                ik = inotblank
                lines.insert(ik+1, "{}={}\n".format(key, value))
# the key line ik will be replaced
            else:
                lines[ik] = "{}={}\n".format(key, value)

#        print("lines:******************")
#        for l in lines:
#            print(l, end='')
#        exit()
        
        if outfile:
            out = tkIniFile(outfile, 'w')
            if out is None:
                return False

            out.WriteLines(lines)
            out.close()
        else:
            out = tkIniFile(self.path, 'w')
            if out is None:
                return False

            out.Rewind()
            out.WriteLines(lines)

        return True

    def GetString(self, section = None, key = None, defstr = None, IsPrint = False):
        return self.get_string(section = section, key = key, def_val = defstr, is_print = IsPrint)
        
    def get_string(self, section = None, key = None, def_val = None, is_print = False):
        re_flag = re.IGNORECASE

        self.Rewind()
        if section is None:
            return self.ReadLines()
        if key is None:
            return self.GetSection(section, is_print = is_print)
        
        self.open(path = self.path, mode = self.mode, IsPrint = is_print)
        if not self.fp:
            if is_print:
                self.errormsg("Error in tkIniFile.GetString(): Can not read [{}]".format(self.path))
            return def_val

        key0 = "^\\[{}\\]".format(section)
        line = self.SkipTo(key0, 0)
        if line is None:
            return def_val
#        print("section header = ", line)

#        key0 = r"^\s*{}\s*=".format(key)
        key0 = r"^\s*{}\s*=\s*(.*)\s*$".format(key)
#        print("key0=[{}]".format(key0))
        while 1:
            line = self.ReadLine()
            if not line:
                return def_val

#            print(f"{line=}")

            match = re.match(r"\s*\[", line)
            if match:
                return def_val
            match = re.search(key0, line, re_flag)
            if match:
                str = match.group(1)
#                print("match:[{}]".format(str))
#                str = re.sub(key0, '', line)
#                str = self.DelCRLF(str)
                return str
        return def_val


    def GetSection(self, section):
        re_flag = re.IGNORECASE
        if section is None:
            return self.ReadLines()

        lines = []
#        self.Open(self.path, self.mode)
#        if not self.fp:
#            self.errormsg(sys._getframe().f_code.co_name, 
#                "Can not read [{}]".format(self.path))
#            return lines

        self.Rewind()
        key0 = "^\\[{}\\]".format(section)
        line = self.SkipTo(key0, 0)
        if line is None:
            return ''
#        print("section header = ", line)

        while 1:
            line = self.ReadLine()
            if line is None:
                return defstr
            match = re.match("\\s*\[", line)
            if match:
                return lines
            line = self.DelCRLF(line)
            lines.append(line)                

        return lines


def main():
    ini = tkIniFile(infile, 'r')

    s = ini.ReadLine()
    print("")
    print("first line=[{}]".format(s))

    title = ini.GetString("Boot", "Title", "none")
    print("")
    print("title=[{}]".format(title))
    sec = ini.GetString("Boot")
    print("")
    print("section [{}]=".format("Boot"), sec)

#    ini.WriteString("EditFile", "fname", "a.txt", outfile)
    print("")
    print("WriteString test")
#    ini.WriteString("EditFile", "fname2", "abc.txt", outfile)
    ini.WriteString("EditFile", "fname2", "abc.txt")

    ini.close()
    

if __name__ == "__main__":
    main()


"""

sub SetIniFile
{
	my ($this, $ProgramPath, $keyhead, $UseIniFileName) = @_;
	$keyhead = 'System/' if(!defined $keyhead);

	if(!$this->{"${keyhead}UseAsIniFile"}) {
		if($UseIniFileName) {
			$this->{"${keyhead}IniFile"} = $ProgramPath;
		}
		return;
	}

	$this->{"SystemKeyHead"} = $keyhead;
	$this->{"$this->{'SystemKeyHead'}ProgramFile"} = $ProgramPath;

#ファイル名を（ベース名, ディレクトリ名, 拡張子）に分解
	my ($drive, $dir, $filename, $ext, $lastdir, $filebody)
			= Deps::SplitFilePath($ProgramPath);
	$dir =~ s/[\\\/]$//;

	$this->{"${keyhead}MyDir"}       = "$drive$dir";
	if($UseIniFileName) {
		$this->{"${keyhead}IniFile"}     = Deps::MakePath("$drive$dir", "$filebody$ext", 0);
	}
	else {
		$this->{"${keyhead}IniFile"}     = Deps::MakePath("$drive$dir", "$filebody.ini", 0);
	}
	$this->{"${keyhead}WinDir"}      = $ENV{'windir'};
	$this->{"${keyhead}TempDir"}     = $ENV{'temp'};
	$this->{"${keyhead}TempDir"}     = $ENV{'TMP_DIR'} unless($this->{'TempDir'});
	$this->{"${keyhead}CPU"}         = $ENV{'CPU'};
	$this->{"${keyhead}ComSpec"}     = $ENV{'ComSpec'};
	$this->{"${keyhead}ComSpec"}     = $ENV{'SHELL'} unless($this->{'ComSpec'});
	$this->{"${keyhead}MachineType"} = $ENV{'MACHTYPE'};
	$this->{"${keyhead}MachineType"} = 'i386' if($OS eq 'MSWin32');
	$this->{"${keyhead}Vendor"}      = $ENV{'VENDOR'};
	$this->{"${keyhead}Vendor"}      = 'intel' if($OS eq 'MSWin32');
	$this->{"${keyhead}HostName"}    = $ENV{'HOSTNAME'};
	$this->{"${keyhead}HostName"}    = $ENV{'COMPUTERNAME'} unless($this->{'HostName'});
	$this->{"${keyhead}OSType"}      = $ENV{'OSTYPE'};
	$this->{"${keyhead}OSType"}      = $OS unless($this->{'OSType'});
	$this->{"${keyhead}Lang"}        = $ENV{'LANG'};
	$this->{"${keyhead}Lang"}        = '' unless($this->{'Lang'});
if($Debug) {
	print "ProgramFile: ", $this->ProgramFile(), "\n";
	print "MyDir      : ", $this->MyDir(), "\n";
	print "IniFile    : ", $this->IniFile(), "\n";
	print "WinDir     : ", $this->WinDir(), "\n";
}

sub IniFile
{
	my ($this) = @_;
	return $this->{"$this->{'SystemKeyHead'}IniFile"};
}

sub ProgramFile
{
	my ($this) = @_;
	return $this->{"$this->{'SystemKeyHead'}ProgramFile"};
}

sub MyDir
{
	my ($this) = @_;
	return $this->{"$this->{'SystemKeyHead'}MyDir"};
}

sub WinDir
{
	my ($this) = @_;
	return $this->{"$this->{'SystemKeyHead'}WinDir"};
}

sub TempDir
{
	my ($this) = @_;
	my $path = $this->{"$this->{'SystemKeyHead'}TempDir"};
	return $path         if($path);
	return $ENV{temp}    if($ENV{temp});
	return $ENV{TMP_DIR} if($ENV{'TMP_DIR'});
}

sub Shell
{
	my ($this) = @_;
	return $this->{"$this->{'SystemKeyHead'}ComSpec"};
}

sub ComSpec
{
	my ($this) = @_;
	return $this->Shell();
}

sub HostName
{
	my ($this) = @_;
	return $this->{"$this->{'SystemKeyHead'}HostName"};
}

sub DeleteAllArrayItems
{
	my ($this, $key) = @_;
	for(my $i = 0 ; ; $i++) {
		last if(!defined $this->{"$key$i"});
		delete $this->{"$key$i"};
	}
}

sub nArrayItems
{
	my ($this, $key) = @_;
	for(my $i = 0 ; ; $i++) {
		if(!defined $this->{"$key$i"}) {
			return $i;
		}
	}
	return 0;
}

sub GetArray
{
	my ($this, $key) = @_;
	my @a;
	for(my $i = 0 ; ; $i++) {
		if(!defined $this->{"$key$i"}) {
			return @a;
		}
		$a[$i] = $this->{"$key$i"};
	}
}

sub AddArrayItems
{
	my ($this, $key, @a) = @_;
	my $n = $this->nArrayItems();
	for(my $i = 0 ; $i < @a ; $i++) {
		my $k = $n + $i;
		$this->{"$key$k"} = $a[$i];
	}
}

sub WriteAll
{
	my ($this, $path, $separator) = @_;
	if($path) {
		$this->SetIniFile($path, "System/", 1);
	}
	else {
		$path = $this->IniFile();
	}
	$separator = $this->{"$this->{'SystemKeyHead'}Separator"} if(!$separator);
	$separator = '=' if(!$separator);
	$this->{"$this->{'SystemKeyHead'}Separator"} = $separator;
	my $keyhead = $this->{'SystemKeyHead'};

#print "write to [", $this->IniFile(), "]\n";
	my $outfile = new JFile();
	my $ret = $outfile->Open($this->IniFile(), "w");
#print "ret=$ret File[", $this->IniFile(), "]\n";
	return undef if(!$ret);

	for my $key (sort keys %$this) {
		next if($key =~ /^$keyhead/i);
		next if($key =~ /^SystemKeyHead$/i);
		next if($key =~ /\//);

		my $val = $this->{$key};
		$outfile->print("$key$separator$val\n");
	}

	my $section = '';
	for my $key (sort keys %$this) {
		next if($key =~ /^$keyhead/i);
		next if($key =~ /^SystemKeyHead$/i);
		next if($key !~ /\//);

		my $val = $this->{$key};
		my ($sec, $key1) = ($key =~ /^([^\/]*)\/(.*)$/);
#print "sec: $sec, $key1\n";
		if($sec ne $section) {
			$section = $sec;
			$outfile->print("\n");
			$outfile->print("[$sec]\n");
		}
		$outfile->print("$key1$separator$val\n");
	}

	$outfile->Close();
	return 1;
}

sub ReadAll
{
	my ($this, $path, $separator, $RemoveQuotation) = @_;
	$RemoveQuotation = 0 if(!defined $RemoveQuotation);

	if($path) {
#print "path=$path\n";
		$this->SetIniFile($path, "System/", 1);
	}
	$separator = $this->{"$this->{'SystemKeyHead'}Separator"} if(!$separator);
	$separator = '=' if(!$separator);
	$this->{"$this->{'SystemKeyHead'}Separator"} = $separator;

	my $seqnum = 1;
	my $infile = new JFile();
	my $ret = $infile->Open($this->IniFile(), "r");
	return undef if(!$ret);
	my $section = '';
	while(!$infile->eof()) {
		my $line = $infile->ReadLine();
		last if(!defined $line);
		Utils::DelSpace($line);
		next if($line eq '');

		if($line =~ /^\[(.*)\]/) {
			$section = $1;
#print "section=$section\n";
			next;
		}
		my ($key, $val) = ($line =~ /^([^$separator]*)$separator(.*?)$/);
		if($key) {
		}
		else {
			$key = "Line$seqnum";
			$val = $line;
			$seqnum++;
		}
		if($section ne '') {
			$key = "$section/$key";
		}
		if($RemoveQuotation) {
			Utils::DelQuote($val);
		}
		$this->{$key} = $val;
#print "Read [$key]: $val\n";
	}
	$infile->Close();

	return 1;
}


"""
