import sys
import numpy as np
import matplotlib.pyplot as plt


# y = w0 + w1 * x

w_range = [-3.0, 3.0]
w_step  = 0.1

w_0 = [0.0, 0.0]
s_0 = 5.0

x_range = [-3.0, 3.0]
x_step  = 0.1

n_add = 10

fontsize = 16



argv = sys.argv
narg = len(argv)
if narg >= 2:
    w_0[0] = float(argv[1])
if narg >= 3:
    w_0[1] = float(argv[2])
if narg >= 4:
    s_0  = float(argv[3])
if narg >= 5:
    n_add  = int(argv[4])


print("")
print(f"w[0] = {w_0[0]}")
print(f"w[1] = {w_0[1]}")
print(f"sigma= {s_0}")
print(f"n_add= {n_add}")

x = np.arange(*w_range, w_step)
y = np.arange(*w_range, w_step)
wx, wy = np.meshgrid(x, y)
probability = np.exp(-((wx - w_0[0]) / s_0)**2 - ((wy -  w_0[1]) / s_0)**2)

x_plot = np.arange(*x_range, x_step)

fig, axes = plt.subplots(1, 2, figsize = (8, 4))
ax1 = axes[0]
ax2 = axes[1]
ax1.tick_params(labelsize = fontsize)
ax2.tick_params(labelsize = fontsize)

#ax1.contour(wx, wy, probability)
ax1.contourf(wx, wy, np.log(probability), levels = 100, cmap = 'OrRd')
#ax1.colorbar(label = "contour level")  
ax1.set_xlabel("$w_x$", fontsize = fontsize)
ax1.set_ylabel("$w_y$", fontsize = fontsize)
ax1.set_aspect('equal')

ax2.set_xlabel("$x$", fontsize = fontsize)
ax2.set_ylabel("$y$", fontsize = fontsize)

plt.tight_layout()
plt.pause(0.0001)


def onclick(event):
    xe, ye  = event.xdata, event.ydata
    print(f"clicked at x={xe}  y={ye}")
#正規分布乱数
    wx_rand = np.random.normal(loc = w_0[0], scale = s_0, size = n_add)
    wy_rand = np.random.normal(loc = w_0[0], scale = s_0, size = n_add)
    print(f"    random: ({wx_rand}, {wy_rand})")
    axes[0].plot(wx_rand, wy_rand, linestyle = '', marker = '.',
            markersize = 3.0, markerfacecolor = 'black', markeredgecolor = 'black', markeredgewidth = 0.01)

    for i in range(n_add):
        y_plot = wx_rand[i] + wy_rand[i] * x_plot
#       axes[1].plot(x_plot, y_plot, linestyle = '-', color = 'black', linewidth = 0.3)
#        axes[1].plot(x_plot, y_plot, linestyle = '-', color = 'black', linewidth = 0.3,
#                                 marker = '.', markersize = 1.0)
        axes[1].plot(x_plot, y_plot, linestyle = '', marker = '.', 
                markersize = 1.0, markerfacecolor = 'black', markeredgecolor = 'black', markeredgewidth = 0.01)

    plt.tight_layout()
    plt.pause(0.0001)

fig.canvas.mpl_connect("button_press_event", onclick)


print("Press ENTER to terminate > ")
input()
