import sys
import csv
import numpy as np
from numpy import exp, log, sin, cos, tan, arcsin, arccos, arctan, sinh, cosh, tanh, sqrt
import pandas as pd
from scipy.optimize import minimize
import matplotlib.pyplot as plt


#nelder-mead    Downhill simplex
#powell         Modified Powell
#cg             conjugate gradient (Polak-Ribiere method)
#bfgs           BFGS法
#newton-cg      Newton-CG
#trust-ncg      信頼領域 Newton-CG 法
#dogleg         信頼領域 dog-leg 法
#L-BFGS-B’ (see here)
#TNC’ (see here)
#COBYLA’ (see here)
#SLSQP’ (see here)
#trust-constr’(see here)
#dogleg’ (see here)
#trust-exact’ (see here)
#trust-krylov’ (see here)
method = "nelder-mead"


h = 0.01

maxiter = 100
tol = 1.0e-5

#==========================================
# Source parameters to be fitted
#==========================================
func = '2.0 * (x[0] - 3.0)**2 + 2.0'
x0_list = "0.0"


argv = sys.argv
n = len(argv)
if n >= 2:
    method = argv[1]
if n >= 3:
    func = argv[2]
if n >= 4:
    x0_list = argv[3]
if n >= 5:
    h = float(argv[4])


def minimize_func(x):
#    print("globals=", globals())
#    print("x=", x)

    y = eval(func, globals(), {"x": x})
    return y

def diff1(x):
    xx = x.copy()
    xx[0] = x[0] - h
    ym = minimize_func(xx)
    xx[0] = x[0] + h
    yp = minimize_func(xx)
    diff = (yp - ym) / 2.0 / h
    return [diff]

iter = 0
def callback(xk):
    global iter
    
    fmin = minimize_func(xk)
    print(f"callback {iter}: xk={xk}  func={fmin}")
    iter += 1

class call_back():
    def __init__(self):
        self.iter = 0
    def __call__(self, xk):
        fmin = minimize_func(xk)
        print(f"callback {self.iter}: xk={xk}  func={fmin}")
        self.iter += 1

        return fmin

#==========================================
# Main routine
#==========================================
def main():
    global x0_list, tol, maxiter

    cb = call_back()

    x0s = [float(s) for s in x0_list.split(',')]

    print("")
    print("Mimize given function")
    print(f"method {method}")
    print(f"func {func}")
    print(f"initial values {x0s}")
    print(f"tol {tol}")

    print("")
    print("Minimize:")
    res = minimize(minimize_func, x0s, jac = diff1, method = method, tol = tol, 
                callback = cb, 
#                callback = callback, 
                options = {'maxiter':maxiter, "disp":True})
#    print("")
#    print(res)
    print("")
    print(f"Function [{func}] takes the minimum at y={res.fun} with x0={res.x}")


if __name__ == "__main__":
    main()
